import { jsx as _jsx, jsxs as _jsxs, Fragment as _Fragment } from "react/jsx-runtime";
import { Box, Divider, Dropdown, IconButton, ListItemDecorator, Menu, MenuButton, MenuItem, Sheet, Tab, TabList, Tabs, Tooltip, Typography, useColorScheme, useTheme, } from "@mui/joy";
import { darken } from "@mui/system";
import { nanoid } from "@reduxjs/toolkit";
import { useCallback, useEffect, useMemo, useRef, useState, } from "react";
import { Rnd } from "react-rnd";
import { NerdWindowContext } from "./nerdWindowContext";
function ensureInBounds(x, y, width, height) {
    const screenWidth = window.innerWidth;
    const screenHeight = window.innerHeight;
    let _x = x;
    let _y = y;
    // Ensure x is within bounds
    if (x < 0)
        _x = 0;
    if (x + width > screenWidth)
        _x = screenWidth - width;
    // Ensure y is within bounds
    if (y < 0)
        _y = 0;
    if (y + height > screenHeight)
        _y = screenHeight - height;
    return [_x, _y];
}
function centerInScreen(width, height) {
    const screenWidth = window.innerWidth;
    const screenHeight = window.innerHeight;
    const _x = (screenWidth - width) / 2;
    const _y = (screenHeight - height) / 2;
    return [_x, _y];
}
// Provider component
export const NerdWindowProvider = ({ children }) => {
    const [fileIndexForMergedView, setFileIndexForMergedView] = useState(0);
    const [merged, setMerged] = useState(localStorage.getItem("nerd-window-merged") === "true");
    const [seeThrough, setSeeThrough] = useState(localStorage.getItem("nerd-window-see-through") === "true");
    const [dragging, setDragging] = useState(false);
    const [nerdWindows, setFiles] = useState([]);
    const [nerdWindowOrder, setFilesOrder] = useState([]);
    const openedFiles = useMemo(() => nerdWindows.filter((f) => !f.collapsed), [nerdWindows]);
    const collapsedFiles = useMemo(() => nerdWindows.filter((f) => f.collapsed), [nerdWindows]);
    const nerdWindowsRef = useRef(nerdWindows);
    const collapsedFilesRef = useRef(collapsedFiles);
    const mergedRef = useRef(merged);
    const theme = useTheme();
    const { mode, systemMode } = useColorScheme();
    const isDark = mode == "dark" || (mode == "system" && systemMode == "dark");
    const updateFile = (newFile) => {
        setFiles((nerdWindows) => {
            const _nerdWindows = Array.from(nerdWindows);
            const index = _nerdWindows.findIndex((e) => e.id === newFile.id);
            _nerdWindows[index] = newFile;
            return _nerdWindows;
        });
    };
    const collapseAll = () => {
        setFiles((nerdWindows) => {
            const _nerdWindows = Array.from(nerdWindows);
            _nerdWindows.forEach((f) => (f.collapsed = true));
            return _nerdWindows;
        });
    };
    const openAll = () => {
        setFiles((nerdWindows) => {
            const _nerdWindows = Array.from(nerdWindows);
            _nerdWindows.forEach((f) => (f.collapsed = false));
            return _nerdWindows;
        });
    };
    const open = useCallback((nerdWindow) => {
        console.log("open");
        const id = nanoid();
        const localStoragePositionString = window.localStorage.getItem("nerd-window-position");
        const localStorageDimensionString = window.localStorage.getItem("nerd-window-dimension");
        let width = nerdWindow.width ?? 500;
        let height = nerdWindow.height ?? 800;
        const [_x, _y] = centerInScreen(width, height);
        let x = nerdWindow.x ?? _x;
        let y = nerdWindow.y ?? _y;
        try {
            const localStoragePosition = JSON.parse(localStoragePositionString ?? "{}");
            const localStorageDimension = JSON.parse(localStorageDimensionString ?? "{}");
            if (mergedRef.current &&
                localStoragePosition?.x !== undefined &&
                localStoragePosition?.y !== undefined &&
                localStorageDimension?.width !== undefined &&
                localStorageDimension?.height !== undefined) {
                x = localStoragePosition.x;
                y = localStoragePosition.y;
                width = localStorageDimension.width;
                height = localStorageDimension.height;
            }
        }
        catch (error) {
            console.error(error);
        }
        const [__x, __y] = ensureInBounds(x, y, width, height);
        x = __x;
        y = __y;
        const fileWithSameDimension = nerdWindowsRef.current.filter((f) => f.width === width && f.height === height);
        setFiles([
            ...nerdWindowsRef.current,
            {
                ...nerdWindow,
                id,
                x: fileWithSameDimension.length > 0 && !mergedRef.current
                    ? x + 20 * fileWithSameDimension.length
                    : x,
                y: fileWithSameDimension.length > 0 && !mergedRef.current
                    ? y + 20 * fileWithSameDimension.length
                    : y,
                width,
                height,
            },
        ]);
        setFilesOrder((order) => {
            const newOrder = [...order];
            newOrder.push(id);
            return newOrder;
        });
        if (mergedRef.current && collapsedFilesRef.current.length) {
            openAll();
        }
        setFileIndexForMergedView(nerdWindowsRef.current.length);
    }, [setFiles, setFileIndexForMergedView, setFilesOrder, openAll]);
    const onDragStart = (nerdWindow) => () => {
        setDragging(true);
        const _nerdWindowOrder = JSON.parse(JSON.stringify(nerdWindowOrder));
        const index = _nerdWindowOrder.findIndex((e) => e === nerdWindow.id);
        _nerdWindowOrder.splice(index, 1);
        if (nerdWindow.id)
            _nerdWindowOrder.push(nerdWindow.id);
        setFilesOrder(_nerdWindowOrder);
    };
    const onDragStop = (nerdWindow) => (_e, d) => {
        setDragging(false);
        updateFile({ ...nerdWindow, x: d.x, y: d.y });
        window.localStorage.setItem("nerd-window-position", JSON.stringify({ x: d.x, y: d.y }));
    };
    const onResizeStart = (nerdWindow) => () => {
        const _nerdWindowOrder = JSON.parse(JSON.stringify(nerdWindowOrder));
        const index = _nerdWindowOrder.findIndex((e) => e === nerdWindow.id);
        _nerdWindowOrder.splice(index, 1);
        if (nerdWindow.id)
            _nerdWindowOrder.push(nerdWindow.id);
        setFilesOrder(_nerdWindowOrder);
    };
    const onResizeStop = (nerdWindow) => (_e, _direction, ref) => {
        const _width = parseInt(ref.style.width.replace("px", ""));
        const _height = parseInt(ref.style.height.replace("px", ""));
        updateFile({
            ...nerdWindow,
            width: _width,
            height: _height,
        });
        window.localStorage.setItem("nerd-window-dimension", JSON.stringify({ width: _width, height: _height }));
    };
    const windowStyle = (nerdWindow, index) => ({
        background: isDark ? "#212121" : "white",
        opacity: seeThrough ||
            nerdWindow.seeTrough ||
            nerdWindow.seeTroughHover ||
            (merged && (nerdWindows[0].seeTrough || nerdWindows[0].seeTroughHover))
            ? 0.8
            : 1,
        transition: "opacity: .3s, boxShadow .3s",
        zIndex: index !== -1
            ? (theme.zIndex.modal + index + 1).toString()
            : theme.zIndex.modal + 1,
        borderRadius: "8px",
        display: "flex",
        flexDirection: "column",
        boxShadow: index !== nerdWindowOrder.length - 1
            ? "rgba(0, 0, 0, 0.1) 0px 4px 10px 4px, rgba(50, 50, 93, 0.25) 0px 8px 12px -8px"
            : "rgba(0, 0, 0, 0.1) 0px 18px 36px 18px, rgba(50, 50, 93, 0.25) 0px 30px 45px -30px",
    });
    useEffect(() => {
        const body = document.getElementsByTagName("body")[0];
        if (body) {
            if (openedFiles.length) {
                body.style.overflow = "hidden";
            }
            else {
                body.style.overflow = "visible";
            }
        }
        return () => {
            const body = document.getElementsByTagName("body")[0];
            if (body)
                body.style.overflow = "visible";
        };
    }, [openedFiles]);
    useEffect(() => {
        nerdWindowsRef.current = nerdWindows;
    }, [nerdWindows]);
    useEffect(() => {
        mergedRef.current = merged;
    }, [merged]);
    useEffect(() => {
        collapsedFilesRef.current = collapsedFiles;
    }, [collapsedFiles]);
    return (_jsxs(NerdWindowContext.Provider, { value: { open }, children: [children, !(merged && collapsedFiles.length) ? (_jsx(_Fragment, { children: (merged && openedFiles.length
                    ? [openedFiles[openedFiles.length - 1]]
                    : openedFiles).map((nerdWindow) => {
                    const index = nerdWindow.id
                        ? nerdWindowOrder.findIndex((v) => v === nerdWindow.id)
                        : 0;
                    return (_jsxs(Rnd, { onDragStart: onDragStart(nerdWindow), onDragStop: onDragStop(nerdWindow), onResizeStart: onResizeStart(nerdWindow), onResizeStop: onResizeStop(nerdWindow), default: {
                            x: nerdWindow.x,
                            y: nerdWindow.y,
                            width: nerdWindow.width,
                            height: nerdWindow.height,
                        }, style: windowStyle(nerdWindow, index), bounds: "window", dragHandleClassName: "draghandle", children: [_jsxs(Box, { sx: {
                                    display: "flex",
                                    justifyContent: "space-between",
                                    gap: 1,
                                    alignItems: "center",
                                }, children: [_jsxs(Box, { sx: {
                                            display: "flex",
                                            p: 1,
                                            gap: 1,
                                            alignItems: "center",
                                        }, children: [_jsx(WindowButton, { onClick: () => {
                                                    if (merged) {
                                                        setFiles([]);
                                                        setFileIndexForMergedView(0);
                                                    }
                                                    else {
                                                        setFiles((u) => u.filter((e) => e.id !== nerdWindow.id));
                                                        setFilesOrder((order) => order.filter((o) => o !== nerdWindow.id));
                                                    }
                                                }, icon: "fa-solid fa-xmark", color: "#FF5F57" }), _jsx(WindowButton, { onClick: () => {
                                                    if (merged) {
                                                        collapseAll();
                                                    }
                                                    else {
                                                        updateFile({ ...nerdWindow, collapsed: true });
                                                        const _nerdWindowOrder = JSON.parse(JSON.stringify(nerdWindowOrder));
                                                        const index = _nerdWindowOrder.findIndex((e) => e === nerdWindow.id);
                                                        _nerdWindowOrder.splice(index, 1);
                                                        setFilesOrder(_nerdWindowOrder);
                                                    }
                                                }, icon: "fa-solid fa-minus", color: "#FFBB2E" })] }), _jsx(Box, { className: "draghandle", flex: 1, sx: {
                                            cursor: "move !important",
                                            height: 30,
                                            textAlign: "center",
                                            display: "flex",
                                            alignItems: "center",
                                            justifyContent: "center",
                                        }, children: _jsx(Typography, { level: "body-sm", children: merged ? `${nerdWindows.length} Items` : nerdWindow.name }) }), _jsxs(Box, { sx: { display: "flex" }, children: [!seeThrough ? (_jsx(Tooltip, { size: "sm", arrow: true, title: "See through", children: _jsx(IconButton, { color: nerdWindow.seeTrough ? "primary" : "neutral", size: "sm", onClick: () => {
                                                        updateFile({
                                                            ...nerdWindow,
                                                            seeTrough: !(nerdWindow.seeTrough ?? false),
                                                        });
                                                    }, onMouseEnter: () => {
                                                        updateFile({
                                                            ...nerdWindow,
                                                            seeTroughHover: true,
                                                        });
                                                    }, onMouseLeave: () => {
                                                        updateFile({
                                                            ...nerdWindow,
                                                            seeTroughHover: false,
                                                        });
                                                    }, children: _jsx("i", { className: "fa-solid fa-eye" }) }) })) : ([]), _jsx(WindowMenu, { collapseAll: collapseAll, merged: merged, openAll: openAll, seeThrough: seeThrough, setFiles: setFiles, setMerged: setMerged, setSeeThrough: setSeeThrough })] })] }), index !== nerdWindowOrder.length - 1 ? (_jsx("div", { style: {
                                    position: "absolute",
                                    top: 40,
                                    left: 0,
                                    width: "100%",
                                    height: "calc(100% - 55px)",
                                    background: "transparent",
                                }, onClick: () => {
                                    const _nerdWindowOrder = JSON.parse(JSON.stringify(nerdWindowOrder));
                                    const index = _nerdWindowOrder.findIndex((e) => e === nerdWindow.id);
                                    _nerdWindowOrder.splice(index, 1);
                                    if (nerdWindow.id)
                                        _nerdWindowOrder.push(nerdWindow.id);
                                    setFilesOrder(_nerdWindowOrder);
                                } })) : null, merged ? (_jsx(Tabs, { onChange: (_e, v) => setFileIndexForMergedView(v), value: fileIndexForMergedView, size: "sm", defaultValue: 0, children: _jsx(TabList, { sx: {
                                        overflow: "auto",
                                        scrollSnapType: "x mandatory",
                                        "&::-webkit-scrollbar": { display: "none" },
                                    }, children: nerdWindows.map((f, i) => (_jsx(Box, { sx: { display: "flex" }, children: _jsx(Tab, { value: i, children: _jsxs(Box, { sx: {
                                                    flexShrink: 0,
                                                    display: "flex",
                                                    gap: 1,
                                                    textOverflow: "ellipsis",
                                                    maxWidth: 120,
                                                }, children: [_jsx(Tooltip, { arrow: true, enterDelay: 1000, enterNextDelay: 1000, size: "sm", title: f.name, children: _jsx(Typography, { noWrap: true, sx: { flexGrow: 1 }, children: f.name }) }), _jsx(Box, { sx: {
                                                            p: 0.5,
                                                            height: 25,
                                                            width: 25,
                                                            background: "#9e9e9e00",
                                                            display: "flex",
                                                            borderRadius: 8,
                                                            alignItems: "center",
                                                            justifyContent: "center",
                                                            transition: "background .3s",
                                                            "&:hover": {
                                                                background: "#9e9e9eAA",
                                                            },
                                                        }, onClick: (e) => {
                                                            e.stopPropagation();
                                                            e.preventDefault();
                                                            setFileIndexForMergedView((index) => index > 0 ? index - 1 : 0);
                                                            setFiles((u) => {
                                                                const files = u.filter((e) => e.id !== nerdWindow.id);
                                                                const lastFile = {
                                                                    ...files[files.length - 1],
                                                                    x: nerdWindow.x,
                                                                    y: nerdWindow.y,
                                                                };
                                                                files.splice(files.length - 1, 1, lastFile);
                                                                return files;
                                                            });
                                                        }, children: _jsx("i", { className: "fa-solid fa-xmark" }) })] }) }) }, f.id))) }) })) : ([]), (merged ? nerdWindows[fileIndexForMergedView] : nerdWindow)
                                .url ? (_jsx("iframe", { style: {
                                    flexGrow: 1,
                                    pointerEvents: dragging ? "none" : undefined,
                                }, src: merged
                                    ? nerdWindows[fileIndexForMergedView]?.url
                                    : nerdWindow?.url })) : ([]), (merged ? nerdWindows[fileIndexForMergedView] : nerdWindow)
                                .child ? (_jsx(Box, { sx: {
                                    display: "flex",
                                    flexGrow: 1,
                                    position: "relative",
                                }, children: (merged
                                    ? nerdWindows[fileIndexForMergedView]
                                    : nerdWindow).child })) : ([]), _jsx(Box, { sx: {
                                    p: "2px",
                                    fontSize: "11px",
                                    display: "flex",
                                    gap: 1,
                                    alignItems: "center",
                                    justifyContent: "end",
                                }, children: _jsx("i", { style: { transform: "rotate(135deg)" }, className: "fa-solid fa-chevron-up" }) })] }, nerdWindow.id));
                }) })) : ([]), collapsedFiles.length ? (_jsxs(_Fragment, { children: [_jsx(Divider, {}), _jsx(Box, { sx: {
                            display: "flex",
                            position: "absolute",
                            bottom: 0,
                            alignItems: "center",
                        }, children: _jsxs(Dropdown, { children: [_jsx(MenuButton, { variant: "plain", sx: { padding: 0 }, children: _jsx(Sheet, { variant: "solid", color: "neutral", invertedColors: true, sx: {
                                            display: "flex",
                                            borderTopRightRadius: 12,
                                            gap: 1,
                                            alignItems: "center",
                                            maxWidth: "calc(100vw - 80px)",
                                            overflow: "clip",
                                            cursor: "pointer",
                                            height: 40,
                                            minWidth: 40,
                                            pl: 1.5,
                                            transformOrigin: "left bottom",
                                            transition: `transform 167ms cubic-bezier(0.33, 0, 0, 1)`,
                                            "&:hover": {
                                                transform: "scale(1.4)",
                                            },
                                        }, children: _jsx("i", { style: { color: "white" }, className: "fa-solid fa-file" }) }) }), _jsxs(Menu, { size: "sm", children: [merged && collapsedFiles.length ? (_jsxs(MenuItem, { onClick: openAll, children: [collapsedFiles.length, " Items"] })) : (_jsx(_Fragment, { children: collapsedFiles.map((nerdWindow) => (_jsx(MenuItem, { onClick: () => {
                                                    updateFile({ ...nerdWindow, collapsed: false });
                                                    setFilesOrder((order) => {
                                                        const newOrder = [...order];
                                                        if (nerdWindow.id)
                                                            newOrder.push(nerdWindow.id);
                                                        return newOrder;
                                                    });
                                                }, children: nerdWindow.name }, nerdWindow.id))) })), _jsx(Divider, {}), _jsxs(MenuItem, { onClick: openAll, children: ["Open all", " ", _jsx(ListItemDecorator, { children: _jsx("i", { className: "fa-solid fa-arrow-up-right" }) })] })] })] }) })] })) : ([])] }));
};
function WindowMenu({ setMerged, merged, setSeeThrough, seeThrough, collapseAll, openAll, setFiles, }) {
    return (_jsxs(Dropdown, { children: [_jsx(MenuButton, { variant: "plain", size: "sm", children: _jsx("i", { className: "fa-solid fa-ellipsis-vertical" }) }), _jsxs(Menu, { size: "sm", sx: { zIndex: 99999 }, children: [_jsxs(MenuItem, { onClick: () => setMerged((e) => {
                            localStorage.setItem("nerd-window-merged", !e ? "true" : "false");
                            return !e;
                        }), children: [_jsx(ListItemDecorator, { children: _jsx("i", { className: merged
                                        ? "fa-duotone fa-solid fa-window-restore"
                                        : "fa-duotone fa-solid fa-window-maximize" }) }), merged ? "Dissociate All Windows" : "Merge All Windows"] }), _jsxs(MenuItem, { onClick: () => {
                            setSeeThrough((e) => {
                                localStorage.setItem("nerd-window-see-through", !e ? "true" : "false");
                                return !e;
                            });
                        }, children: [_jsx(ListItemDecorator, { children: _jsx("i", { className: "fa-solid fa-eye" }) }), seeThrough ? "Opaque Mode" : "See Through Mode"] }), _jsxs(MenuItem, { onClick: openAll, children: [_jsx(ListItemDecorator, { children: _jsx("i", { className: "fa-solid fa-arrow-up-right" }) }), "Open All"] }), _jsxs(MenuItem, { onClick: collapseAll, children: [_jsx(ListItemDecorator, { children: _jsx("i", { className: "fa-solid fa-arrow-down-left" }) }), "Collapse All"] }), _jsxs(MenuItem, { onClick: () => setFiles([]), children: [_jsx(ListItemDecorator, { children: _jsx("i", { className: "fa-solid fa-xmark" }) }), "Close All"] }), _jsxs(MenuItem, { onClick: () => {
                            localStorage.removeItem("nerd-window-position");
                            localStorage.removeItem("nerd-window-dimension");
                            localStorage.removeItem("nerd-window-see-through");
                            localStorage.removeItem("nerd-window-merged");
                        }, children: [_jsx(ListItemDecorator, { children: _jsx("i", { className: "fa-solid fa-broom" }) }), "Reset Local Storage"] })] })] }));
}
function WindowButton({ icon, color, onClick, }) {
    return (_jsx(Box, { onClick: onClick, sx: {
            width: 15,
            height: 15,
            background: color,
            display: "flex",
            alignItems: "center",
            borderRadius: 4,
            justifyContent: "center",
            color: darken(color, 0.5),
            fontSize: 10,
            cursor: "pointer",
        }, children: _jsx("i", { className: icon }) }));
}
